# This scripts does the following:
#   - Create a  YAML File ( kaniko-demo-production.yaml ) which can  be used redeploy the APP
#   - Deploy the APP a first time to our proction system
#   - Create namepaces and serviceaccounts to restrict Kluster access to the namespace ONLY
#
# Gitlab provided parameter - we need to set theus parameter manualy to get the proper results
# If you run this build process via Gitlab pipelines you don't need to configure these parameters
#
# For the  production deployment you only the following 4 values must to set/changed to reflect your env
#   Note:  SERVICE-PORT and INGRESS_HOST must be unique at your kubernetes cluster level - so be carefull !
#          CI_PROJECT_NAME must be copied from your GITLAB project
#          CI_REGISTRY_NAME must match your harbor configuration
#
#   In any case we expect that you have create images like 
#      - harbor.informatik.fh-nuernberg.de/hutzler/kaniko-demo-production     
#
CI_PROJECT_NAME=kaniko-demo
SERVICE_PORT=9604
INGRESS_HOST="kube-master-h4.informatik.fh-nuernberg.de" 
CI_REGISTRY_NAME="harbor.informatik.fh-nuernberg.de/hutzler"
#
#
# Normally you dont need to change the values below
#  
CI_ENVIRONMENT_NAME=production
CI_REGISTRY_IMAGE=$CI_REGISTRY_NAME"/"$CI_PROJECT_NAME"-"$CI_ENVIRONMENT_NAME
POD_PORT=80
export KUBE_NAMESPACE=${CI_PROJECT_NAME}-${CI_ENVIRONMENT_NAME}
IMAGE_ESCAPED=$(echo $CI_REGISTRY_IMAGE  | sed -r -e 's/\//\\\//g')
echo "Harbor Image to be deployed: "  $CI_REGISTRY_IMAGE
#
#
# Resources Naming Stuff - you don' need to change anythong
#
APP_NAME=${CI_PROJECT_NAME}-app
SERVICE_NAME=${CI_PROJECT_NAME}-svc
POD_NAME=${CI_PROJECT_NAME}-pod
INGRESS_NAME=${CI_PROJECT_NAME}-ingress
export SERVICE_ACCOUNT=${KUBE_NAMESPACE}-service-account
YAML_FILE=${CI_PROJECT_NAME}"-"${CI_ENVIRONMENT_NAME}.yaml


# Delete the namespace
kubectl delete namespace ${KUBE_NAMESPACE}

# Create a YAML File for documentation purposes how role and rolebinding resoruces are build
envsubst < create-svc-acc-template.yaml > DEBUG/create-svc-acc-${CI_PROJECT_NAME}"-"${CI_ENVIRONMENT_NAME}.yaml

# Finally create roles,namespace,rolebinding resources
envsubst < create-svc-acc-template.yaml | kubectl apply -f -

# Validate namespace and serviceaccount creation 
kubectl get namespace  ${KUBE_NAMESPACE}
kubectl get serviceaccounts  -n ${KUBE_NAMESPACE}

#
# Create a new  service account and switch to that svc  account before deploying the APP
#
TOKENNAME=`kubectl -n   ${KUBE_NAMESPACE} get serviceaccount/${SERVICE_ACCOUNT} -o jsonpath='{.secrets[0].name}'`
echo $TOKENNAME
TOKEN=`kubectl -n   ${KUBE_NAMESPACE} get secret $TOKENNAME -o jsonpath='{.data.token}'| base64 --decode`
kubectl config set-credentials ${SERVICE_ACCOUNT} --token=$TOKEN
kubectl config set-context --current --user=${SERVICE_ACCOUNT} 
echo -n "Can-i create deployments: "; kubectl auth can-i create deployments -n ${KUBE_NAMESPACE}
echo -n "Can-i create services: "; kubectl auth can-i create services -n ${KUBE_NAMESPACE}
echo -n "Can-i create ingress: "; kubectl auth can-i create ingress -n ${KUBE_NAMESPACE}

# If you enable the line below you get expected error: 
#        Error from server (Forbidden): namespaces is forbidden: 
#        User "system:serviceaccount:kaniko-demo-production:kaniko-demo-production-service-account" cannot list resource "namespaces" in API group "" at the cluster scope
# kubectl get namespace  

# We only should see our  newly create serviceaccounts
kubectl get serviceaccounts  -n   ${KUBE_NAMESPACE}

# Run customiue to build new YAML File for deployment,service and ingress resource 
kustomize build ../overlays/${CI_ENVIRONMENT_NAME} | sed "s/__SERVICE_NAME__/${SERVICE_NAME}/" | sed "s/__APP_NAME__/${APP_NAME}/"   \
       | sed "s/__SERVICE_PORT__/${SERVICE_PORT}/" \
       | sed "s/__POD_NAME__/${POD_NAME}/" | sed "s/__POD_PORT__/${POD_PORT}/" \
       | sed "s/__IMAGE__/${IMAGE_ESCAPED}/" | sed "s/__INGRESS_NAME__/${INGRESS_NAME}/" \
       | sed "s/__INGRESS_HOST__/${INGRESS_HOST}/" \
       | sed "s/__SERVICE_ACCOUNT__/${SERVICE_ACCOUNT}/"  > DEBUG/$YAML_FILE

# Deploy the resources 
kubectl apply -f  DEBUG/$YAML_FILE  -n $KUBE_NAMESPACE
sleep 5

curl $INGRESS_HOST


# This is very important swith back to default context 
kubectl config set-context --current --user=default
